CREATE TABLE IF NOT EXISTS `Realms`
(
    `RealmId`        BIGINT       NOT NULL AUTO_INCREMENT,
    `Name`           VARCHAR(255) NOT NULL,
    `RoleId`         BIGINT       NOT NULL,
    `ServerId`       BIGINT,
    `CreatedAt`      DATETIME(6)  NULL DEFAULT CURRENT_TIMESTAMP(6),
    `LastModifiedAt` DATETIME(6)  NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
    PRIMARY KEY (`RealmId`),
    FOREIGN KEY (`ServerId`) REFERENCES `Servers` (`ServerId`)
);


CREATE TABLE IF NOT EXISTS `RealmsHistory`
(
    `RealmId`  BIGINT(20)   NOT NULL,
    `Name`     VARCHAR(255) NOT NULL,
    `RoleId`   BIGINT       NOT NULL,
    `ServerId` BIGINT(20) DEFAULT NULL,
    `Deleted`  BOOL       DEFAULT FALSE,
    `DateFrom` DATETIME(6)  NOT NULL,
    `DateTo`   DATETIME(6)  NOT NULL
);

DROP TRIGGER IF EXISTS `TR_RealmsUpdate`;

CREATE TRIGGER `TR_RealmsUpdate`
    AFTER UPDATE
    ON `Realms`
    FOR EACH ROW
BEGIN
    INSERT INTO `RealmsHistory` (`RealmId`, `Name`, `RoleId`, `ServerId`, `DateFrom`, `DateTo`)
    VALUES (OLD.RealmId, OLD.Name, OLD.RoleId, OLD.ServerId, OLD.LastModifiedAt, CURRENT_TIMESTAMP(6));
END;

DROP TRIGGER IF EXISTS `TR_RealmsDelete`;

CREATE TRIGGER `TR_RealmsDelete`
    AFTER DELETE
    ON `Realms`
    FOR EACH ROW
BEGIN
    INSERT INTO `RealmsHistory` (`RealmId`, `Name`, `RoleId`, `ServerId`, `Deleted`, `DateFrom`, `DateTo`)
    VALUES (OLD.RealmId, OLD.Name, OLD.RoleId, OLD.ServerId, TRUE, OLD.LastModifiedAt, CURRENT_TIMESTAMP(6));
END;

-- Realm moderators

CREATE TABLE IF NOT EXISTS `RealmModerators`
(
    `Id`             BIGINT      NOT NULL AUTO_INCREMENT,
    `RealmId`        BIGINT      NOT NULL,
    `UserId`         BIGINT      NOT NULL,
    `ServerId`       BIGINT,
    `CreatedAt`      DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
    `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
    PRIMARY KEY (`Id`),
    FOREIGN KEY (`RealmId`) REFERENCES `Realms` (`RealmId`),
    FOREIGN KEY (`UserId`) REFERENCES `Users` (`UserId`),
    FOREIGN KEY (`ServerId`) REFERENCES `Servers` (`ServerId`)
);


CREATE TABLE IF NOT EXISTS `RealmModeratorsHistory`
(
    `Id`       BIGINT(20)  NOT NULL,
    `RealmId`  BIGINT      NOT NULL,
    `UserId`   BIGINT      NOT NULL,
    `ServerId` BIGINT(20) DEFAULT NULL,
    `Deleted`  BOOL       DEFAULT FALSE,
    `DateFrom` DATETIME(6) NOT NULL,
    `DateTo`   DATETIME(6) NOT NULL
);

DROP TRIGGER IF EXISTS `TR_RealmModeratorsUpdate`;

CREATE TRIGGER `TR_RealmModeratorsUpdate`
    AFTER UPDATE
    ON `RealmModerators`
    FOR EACH ROW
BEGIN
    INSERT INTO `RealmModeratorsHistory` (`Id`, `RealmId`, `UserId`, `ServerId`, `DateFrom`, `DateTo`)
    VALUES (OLD.Id, OLD.RealmId, OLD.UserId, OLD.ServerId, OLD.LastModifiedAt, CURRENT_TIMESTAMP(6));
END;

DROP TRIGGER IF EXISTS `TR_RealmModeratorsDelete`;

CREATE TRIGGER `TR_RealmModeratorsDelete`
    AFTER DELETE
    ON `RealmModerators`
    FOR EACH ROW
BEGIN
    INSERT INTO `RealmModeratorsHistory` (`Id`, `RealmId`, `UserId`, `ServerId`, `Deleted`, `DateFrom`, `DateTo`)
    VALUES (OLD.Id, OLD.RealmId, OLD.UserId, OLD.ServerId, TRUE, OLD.LastModifiedAt, CURRENT_TIMESTAMP(6));
END;


